﻿using System;
using System.IO;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Infragistics.Controls.Grids;
using Infragistics.Persistence;

namespace BlotterShared.Utilities
{
    public class XamGridPersistenceSettings
    {
        //The Settings used by all Grids that are saved/loaded
        public static PersistenceSettings GetSettings()
        {
            var ps = new PersistenceSettings
                          {
                              LoadPersistenceOptions =
                                  Infragistics.Persistence.Primitives.PersistenceOption.OnlySpecified,
                              SavePersistenceOptions =
                                  Infragistics.Persistence.Primitives.PersistenceOption.OnlySpecified
                          };
            ps.PropertySettings.Add(new PropertyNamePersistenceInfo { PropertyName = "AllowFixedColumns" });
            ps.PropertySettings.Add(new PropertyNamePersistenceInfo { PropertyName = "AllowFixedColumns" });
            ps.PropertySettings.Add(new PropertyNamePersistenceInfo { PropertyName = "AllowHideColumnIcon" });
            ps.PropertySettings.Add(new PropertyNamePersistenceInfo { PropertyName = "AllowColumnMoving" });
            ps.PropertySettings.Add(new PropertyNamePersistenceInfo { PropertyName = "IsFixed" });
            ps.PropertySettings.Add(new PropertyNamePersistenceInfo { PropertyName = "AllowSorting" });
            ps.PropertySettings.Add(new PropertyNamePersistenceInfo { PropertyName = "IsSorted" });
            ps.PropertySettings.Add(new PropertyNamePersistenceInfo { PropertyName = "FixedColumnsLeft" });
            ps.PropertySettings.Add(new PropertyNamePersistenceInfo { PropertyName = "FixedColumnsRight" });
            ps.PropertySettings.Add(new PropertyNamePersistenceInfo { PropertyName = "AllowFiltering" });
            ps.PropertySettings.Add(new PropertyNamePersistenceInfo { PropertyName = "PagerSettings.PageSize" });
            ps.PropertySettings.Add(new PropertyNamePersistenceInfo { PropertyName = "GroupBySettings", Options = Infragistics.Persistence.Primitives.PropertyNamePersistenceOptions.Contains });
            ps.PropertySettings.Add(new PropertyNamePersistenceInfo { PropertyName = "ColumnLayouts[].Columns[].Visibility", Options = Infragistics.Persistence.Primitives.PropertyNamePersistenceOptions.PropertyPath });
            
            return ps;

        }

        //Simple converter to convert into a byte[] for saving to the DB
        public static byte[] MemoryStreamToByteArray(MemoryStream input)
        {
            var memoryStream = new MemoryStream();
            input.CopyTo(memoryStream);
            return memoryStream.ToArray();
        }

        //The method that actually applies the settings. 
        //The checks for length > 4 are kind of redundant now, 
        //as I thought it could be an issue with applying an empty Stream with array length 4 (minimum possible size for instantiated Byte[]
        //but it still had the issue when applying verified data with an expected length

        public static void ApplyGridSettings(Byte[] settingsAsByte, MemoryStream defaultSettings,  XamGrid grid, PersistenceSettings ps)
        {
            Stream settings = null;
            if (settingsAsByte != null && settingsAsByte.Length > 4)
            {
                settings = new MemoryStream(settingsAsByte);
            }
            else if(defaultSettings != null && defaultSettings.Length > 4)
            {
                settings = defaultSettings;
            }
            if (settings != null)
            {
                settings.Position = 0;
                PersistenceManager.Load(grid, settings, ps);
            }

        }

    }

      //External code using these methods

      //This is the method used in the View.cs to pass the Stream information to the above methods
      //When the data arrives and the Grid Settings are received in the View.cs, if the grid has not yet loaded it saves a temprary local version of the settings
      //and only applies them when everything is loaded to avoid overriding the settings with a refresh of data.        
      public void ApplyGridSettings(object o)
        {
            if(!_loaded)
            {
                _settingsToLoad = o as Byte[];
            }
            else
            {
                var settingsAsByte = o as Byte[];
                XamGridPersistenceSettings.ApplyGridSettings(settingsAsByte, _defaultSettings, blotterGrid, _ps);
                _settingsToLoad = null;
            }
            
        }

        //This is the method that saves the grid info, and passes it to the viewmodel that saves it to the DB
        private void SnapShotGrid(object o)
        {
            var gridSettings = PersistenceManager.Save(blotterGrid, _ps);
            if (gridSettings != null)
            {
                viewModel.SaveGridSettings(XamGridPersistenceSettings.MemoryStreamToByteArray(gridSettings));
            }
        }
}
